"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const vscode = require("vscode");
const nls = require("vscode-nls");
const extension = require("../src/extension");
const Profiles_1 = require("./Profiles");
const ZoweExplorerApiRegister_1 = require("./api/ZoweExplorerApiRegister");
const utils_1 = require("./utils");
const ZoweTreeProvider_1 = require("./abstract/ZoweTreeProvider");
const ZoweDatasetNode_1 = require("./ZoweDatasetNode");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Creates the Dataset tree that contains nodes of sessions and data sets
 *
 * @export
 */
function createDatasetTree(log) {
    return __awaiter(this, void 0, void 0, function* () {
        const tree = new DatasetTree();
        yield tree.initialize(log);
        yield tree.addSession();
        return tree;
    });
}
exports.createDatasetTree = createDatasetTree;
/**
 * A tree that contains nodes of sessions and data sets
 *
 * @export
 * @class DatasetTree
 * @implements {vscode.TreeDataProvider}
 */
class DatasetTree extends ZoweTreeProvider_1.ZoweTreeProvider {
    constructor() {
        super(DatasetTree.persistenceSchema, new ZoweDatasetNode_1.ZoweDatasetNode(localize(0, null), vscode.TreeItemCollapsibleState.Collapsed, null, null, null));
        this.mSessionNodes = [];
        this.mFavorites = [];
        this.mFavoriteSession.contextValue = extension.FAVORITE_CONTEXT;
        this.mFavoriteSession.iconPath = utils_1.applyIcons(this.mFavoriteSession);
        this.mSessionNodes = [this.mFavoriteSession];
        this.treeView = vscode.window.createTreeView("zowe.explorer", { treeDataProvider: this });
    }
    /**
     * Takes argument of type IZoweDatasetTreeNode and retrieves all of the first level children
     *
     * @param {IZoweDatasetTreeNode} [element] - Optional parameter; if not passed, returns root session nodes
     * @returns {IZoweDatasetTreeNode[] | Promise<IZoweDatasetTreeNode[]>}
     */
    getChildren(element) {
        return __awaiter(this, void 0, void 0, function* () {
            if (element) {
                if (element.contextValue === extension.FAVORITE_CONTEXT) {
                    return this.mFavorites;
                }
                return element.getChildren();
            }
            return this.mSessionNodes;
        });
    }
    /**
     * Initializes the tree based on favorites held in persistent store
     *
     * @param {Logger} log
     */
    initialize(log) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log = log;
            this.log.debug(localize(1, null));
            const lines = this.mHistory.readFavorites();
            for (const line of lines) {
                if (line === "") {
                    continue;
                }
                // validate line
                const favoriteDataSetPattern = /^\[.+\]\:\s[a-zA-Z#@\$][a-zA-Z0-9#@\$\-]{0,7}(\.[a-zA-Z#@\$][a-zA-Z0-9#@\$\-]{0,7})*\{p?ds\}$/;
                const favoriteSearchPattern = /^\[.+\]\:\s.*\{session\}$/;
                if (favoriteDataSetPattern.test(line)) {
                    const sesName = line.substring(1, line.lastIndexOf("]")).trim();
                    try {
                        const profile = Profiles_1.Profiles.getInstance().loadNamedProfile(sesName);
                        const session = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(profile).getSession();
                        let node;
                        if (line.substring(line.indexOf("{") + 1, line.lastIndexOf("}")) === extension.DS_PDS_CONTEXT) {
                            node = new ZoweDatasetNode_1.ZoweDatasetNode(line.substring(0, line.indexOf("{")), vscode.TreeItemCollapsibleState.Collapsed, this.mFavoriteSession, session, undefined, undefined, profile);
                        }
                        else {
                            node = new ZoweDatasetNode_1.ZoweDatasetNode(line.substring(0, line.indexOf("{")), vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, session, undefined, undefined, profile);
                            node.command = { command: "zowe.ZoweNode.openPS", title: "", arguments: [node] };
                        }
                        node.contextValue += extension.FAV_SUFFIX;
                        node.iconPath = utils_1.applyIcons(node);
                        this.mFavorites.push(node);
                    }
                    catch (e) {
                        const errMessage = localize(2, null) + sesName +
                            localize(3, null) +
                            localize(4, null) +
                            utils_1.getAppName(extension.ISTHEIA) + localize(5, null);
                        yield utils_1.errorHandling(e, null, errMessage);
                        continue;
                    }
                }
                else if (favoriteSearchPattern.test(line)) {
                    const sesName = line.substring(1, line.lastIndexOf("]")).trim();
                    let profile;
                    try {
                        profile = Profiles_1.Profiles.getInstance().loadNamedProfile(sesName);
                    }
                    catch (error) {
                        const errMessage = localize(6, null) +
                            +sesName +
                            localize(7, null);
                        yield utils_1.errorHandling(error, null, errMessage);
                        continue;
                    }
                    const session = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(profile).getSession();
                    const node = new ZoweDatasetNode_1.ZoweDatasetNode(line.substring(0, line.lastIndexOf("{")), vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, session, undefined, undefined, profile);
                    node.command = { command: "zowe.pattern", title: "", arguments: [node] };
                    const light = path.join(__dirname, "..", "..", "resources", "light", "pattern.svg");
                    const dark = path.join(__dirname, "..", "..", "resources", "dark", "pattern.svg");
                    node.iconPath = { light, dark };
                    node.contextValue = extension.DS_SESSION_CONTEXT + extension.FAV_SUFFIX;
                    node.iconPath = utils_1.applyIcons(node);
                    this.mFavorites.push(node);
                }
                else {
                    vscode.window.showErrorMessage(localize(8, null) + line);
                }
            }
        });
    }
    /**
     * Returns the tree view for the current DatasetTree
     *
     * @returns {vscode.TreeView<IZoweDatasetTreeNode>}
     */
    getTreeView() {
        return this.treeView;
    }
    /**
     * Adds a new session to the data set tree
     *
     * @param {string} [sessionName] - optional; loads default profile if not passed
     */
    addSession(sessionName) {
        return __awaiter(this, void 0, void 0, function* () {
            // Loads profile associated with passed sessionName, default if none passed
            if (sessionName) {
                const zosmfProfile = Profiles_1.Profiles.getInstance().loadNamedProfile(sessionName);
                if (zosmfProfile) {
                    this.addSingleSession(zosmfProfile);
                }
            }
            else {
                const profiles = Profiles_1.Profiles.getInstance().allProfiles;
                for (const zosmfProfile of profiles) {
                    // If session is already added, do nothing
                    if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === zosmfProfile.name)) {
                        continue;
                    }
                    for (const session of this.mHistory.getSessions()) {
                        if (session === zosmfProfile.name) {
                            this.addSingleSession(zosmfProfile);
                        }
                    }
                }
                if (this.mSessionNodes.length === 1) {
                    this.addSingleSession(Profiles_1.Profiles.getInstance().getDefaultProfile());
                }
            }
            this.refresh();
        });
    }
    /**
     * Removes a session from the list in the data set tree
     *
     * @param {IZoweDatasetTreeNode} [node]
     */
    deleteSession(node) {
        this.mSessionNodes = this.mSessionNodes.filter((tempNode) => tempNode.label.trim() !== node.label.trim());
        let revisedLabel = node.label;
        if (revisedLabel.includes("[")) {
            revisedLabel = revisedLabel.substring(0, revisedLabel.indexOf(" ["));
        }
        this.mHistory.removeSession(revisedLabel);
        this.refresh();
    }
    /**
     * Adds a node to the favorites list
     *
     * @param {IZoweDatasetTreeNode} node
     */
    addFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            let temp;
            if (node.contextValue === extension.DS_MEMBER_CONTEXT) {
                if (node.getParent().contextValue === extension.DS_PDS_CONTEXT + extension.FAV_SUFFIX) {
                    vscode.window.showInformationMessage(localize(9, null));
                    return;
                }
                this.addFavorite(node.getParent());
                return;
            }
            else if (node.contextValue === extension.DS_SESSION_CONTEXT) {
                temp = new ZoweDatasetNode_1.ZoweDatasetNode("[" + node.getSessionNode().label.trim() + "]: " + node.pattern, vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, node.getSession(), node.contextValue, node.getEtag(), node.getProfile());
                temp.contextValue = extension.DS_SESSION_CONTEXT + extension.FAV_SUFFIX;
                temp.iconPath = utils_1.applyIcons(temp);
                // add a command to execute the search
                temp.command = { command: "zowe.pattern", title: "", arguments: [temp] };
            }
            else { // pds | ds
                temp = new ZoweDatasetNode_1.ZoweDatasetNode("[" + node.getSessionNode().label.trim() + "]: " + node.label, node.collapsibleState, this.mFavoriteSession, node.getSession(), node.contextValue, node.getEtag(), node.getProfile());
                temp.contextValue += extension.FAV_SUFFIX;
                if (temp.contextValue === extension.DS_DS_CONTEXT + extension.FAV_SUFFIX) {
                    temp.command = { command: "zowe.ZoweNode.openPS", title: "", arguments: [temp] };
                }
                temp.iconPath = utils_1.applyIcons(temp);
            }
            if (!this.mFavorites.find((tempNode) => (tempNode.label === temp.label) && (tempNode.contextValue === temp.contextValue))) {
                this.mFavorites.push(temp);
                utils_1.sortTreeItems(this.mFavorites, extension.DS_SESSION_CONTEXT + extension.FAV_SUFFIX);
                yield this.updateFavorites();
                this.refreshElement(this.mFavoriteSession);
            }
        });
    }
    /**
     * Renames a node based on the profile and it's label
     *
     * @param {string} profileLabel
     * @param {string} beforeLabel
     * @param {string} afterLabel
     */
    renameNode(profileLabel, beforeLabel, afterLabel) {
        return __awaiter(this, void 0, void 0, function* () {
            const sessionNode = this.mSessionNodes.find((session) => session.label === `${profileLabel} `);
            if (sessionNode) {
                const matchingNode = sessionNode.children.find((node) => node.label === beforeLabel);
                if (matchingNode) {
                    matchingNode.label = afterLabel;
                    this.refreshElement(matchingNode);
                }
            }
        });
    }
    /**
     * Renames a node from the favorites list
     *
     * @param {IZoweDatasetTreeNode} node
     */
    renameFavorite(node, newLabel) {
        return __awaiter(this, void 0, void 0, function* () {
            const matchingNode = this.mFavorites.find((temp) => (temp.label === node.label) && (temp.contextValue.startsWith(node.contextValue)));
            if (matchingNode) {
                const prefix = matchingNode.label.substring(0, matchingNode.label.indexOf(":") + 2);
                matchingNode.label = prefix + newLabel;
                this.refreshElement(matchingNode);
            }
        });
    }
    /**
     * Finds the equivalent node as a favorite
     *
     * @param {IZoweDatasetTreeNode} node
     */
    findFavoritedNode(node) {
        return this.mFavorites.find((temp) => (temp.label === `[${node.getParent().getLabel()}]: ${node.label}`) && (temp.contextValue.includes(node.contextValue)));
    }
    /**
     * Finds the equivalent node not as a favorite
     *
     * @param {IZoweDatasetTreeNode} node
     */
    findNonFavoritedNode(node) {
        const profileLabel = node.label.substring(1, node.label.indexOf("]"));
        const nodeLabel = node.label.substring(node.label.indexOf(":") + 2);
        const sessionNode = this.mSessionNodes.find((session) => session.label.trim() === profileLabel);
        return sessionNode.children.find((temp) => temp.label === nodeLabel);
    }
    /**
     * Removes a node from the favorites list
     *
     * @param {IZoweDatasetTreeNode} node
     */
    removeFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mFavorites = this.mFavorites.filter((temp) => !((temp.label === node.label) && (temp.contextValue.startsWith(node.contextValue))));
            this.refresh();
            yield this.updateFavorites();
            this.refreshElement(this.mFavoriteSession);
        });
    }
    updateFavorites() {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = this.mFavorites.map((fav) => fav.label + "{" + fav.contextValue.substring(0, fav.contextValue.indexOf(extension.FAV_SUFFIX)) + "}");
            this.mHistory.updateFavorites(settings);
        });
    }
    onDidChangeConfiguration(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (e.affectsConfiguration(DatasetTree.persistenceSchema)) {
                const setting = Object.assign({}, vscode.workspace.getConfiguration().get(DatasetTree.persistenceSchema));
                if (!setting.persistence) {
                    setting.favorites = [];
                    setting.history = [];
                    yield vscode.workspace.getConfiguration().update(DatasetTree.persistenceSchema, setting, vscode.ConfigurationTarget.Global);
                }
            }
        });
    }
    addHistory(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mHistory.addHistory(criteria);
            this.refresh();
        });
    }
    getHistory() {
        return this.mHistory.getHistory();
    }
    datasetFilterPrompt(node) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug(localize(10, null));
            let pattern;
            let usrNme;
            let passWrd;
            let baseEncd;
            let sesNamePrompt;
            if (node.contextValue.endsWith(extension.FAV_SUFFIX)) {
                sesNamePrompt = node.label.substring(1, node.label.indexOf("]"));
            }
            else {
                sesNamePrompt = node.label;
            }
            if ((!node.getSession().ISession.user) || (!node.getSession().ISession.password)) {
                try {
                    const values = yield Profiles_1.Profiles.getInstance().promptCredentials(sesNamePrompt);
                    if (values !== undefined) {
                        usrNme = values[0];
                        passWrd = values[1];
                        baseEncd = values[2];
                    }
                }
                catch (error) {
                    yield utils_1.errorHandling(error, node.getProfileName(), localize(11, null) + `datasetFilterPrompt.optionalProfiles!`);
                }
                if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                    node.getSession().ISession.user = usrNme;
                    node.getSession().ISession.password = passWrd;
                    node.getSession().ISession.base64EncodedAuth = baseEncd;
                    this.validProfile = 0;
                }
                else {
                    return;
                }
                yield this.refreshElement(node);
                yield this.refresh();
            }
            else {
                this.validProfile = 0;
            }
            if (this.validProfile === 0) {
                if (node.contextValue === extension.DS_SESSION_CONTEXT) {
                    if (this.mHistory.getHistory().length > 0) {
                        const createPick = new utils_1.FilterDescriptor(DatasetTree.defaultDialogText);
                        const items = this.mHistory.getHistory().map((element) => new utils_1.FilterItem(element));
                        if (extension.ISTHEIA) {
                            const options1 = {
                                placeHolder: localize(12, null)
                            };
                            // get user selection
                            const choice = (yield vscode.window.showQuickPick([createPick, ...items], options1));
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(13, null));
                                return;
                            }
                            pattern = choice === createPick ? "" : choice.label;
                        }
                        else {
                            const quickpick = vscode.window.createQuickPick();
                            quickpick.items = [createPick, ...items];
                            quickpick.placeholder = localize(14, null);
                            quickpick.ignoreFocusOut = true;
                            quickpick.show();
                            const choice = yield utils_1.resolveQuickPickHelper(quickpick);
                            quickpick.hide();
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(15, null));
                                return;
                            }
                            if (choice instanceof utils_1.FilterDescriptor) {
                                if (quickpick.value) {
                                    pattern = quickpick.value;
                                }
                            }
                            else {
                                pattern = choice.label;
                            }
                        }
                    }
                    if (!pattern) {
                        // manually entering a search
                        const options2 = {
                            prompt: localize(16, null),
                            value: node.pattern,
                        };
                        // get user input
                        pattern = yield vscode.window.showInputBox(options2);
                        if (!pattern) {
                            vscode.window.showInformationMessage(localize(17, null));
                            return;
                        }
                    }
                }
                else {
                    // executing search from saved search in favorites
                    pattern = node.label.trim().substring(node.getLabel().indexOf(":") + 2);
                    const session = node.label.trim().substring(node.label.trim().indexOf("[") + 1, node.label.trim().indexOf("]"));
                    yield this.addSession(session);
                    const faveNode = node;
                    node = this.mSessionNodes.find((tempNode) => tempNode.label.trim() === session);
                    if ((!node.getSession().ISession.user) || (!node.getSession().ISession.password)) {
                        node.getSession().ISession.user = faveNode.getSession().ISession.user;
                        node.getSession().ISession.password = faveNode.getSession().ISession.password;
                        node.getSession().ISession.base64EncodedAuth = faveNode.getSession().ISession.base64EncodedAuth;
                    }
                }
                // update the treeview with the new pattern
                node.label = node.label.trim() + " ";
                node.label.trim();
                node.tooltip = node.pattern = pattern.toUpperCase();
                node.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                node.dirty = true;
                node.iconPath = utils_1.applyIcons(node, extension.ICON_STATE_OPEN);
                this.addHistory(node.pattern);
            }
        });
    }
    /**
     * Adds a single session to the data set tree
     *
     */
    addSingleSession(profile) {
        if (profile) {
            // If session is already added, do nothing
            if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === profile.name)) {
                return;
            }
            // Uses loaded profile to create a session with the MVS API
            const session = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(profile).getSession();
            // Creates ZoweDatasetNode to track new session and pushes it to mSessionNodes
            const node = new ZoweDatasetNode_1.ZoweDatasetNode(profile.name, vscode.TreeItemCollapsibleState.Collapsed, null, session, undefined, undefined, profile);
            node.contextValue = extension.DS_SESSION_CONTEXT;
            node.iconPath = utils_1.applyIcons(node);
            this.mSessionNodes.push(node);
            this.mHistory.addSession(profile.name);
        }
    }
}
exports.DatasetTree = DatasetTree;
DatasetTree.persistenceSchema = "Zowe-DS-Persistent";
DatasetTree.defaultDialogText = "\uFF0B " + localize(18, null);

//# sourceMappingURL=DatasetTree.js.map
