"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const nls = require("vscode-nls");
const utils = require("./utils");
const extension = require("./extension");
const ZoweTreeNode_1 = require("./abstract/ZoweTreeNode");
const ZoweExplorerApiRegister_1 = require("./api/ZoweExplorerApiRegister");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * A type of TreeItem used to represent sessions and data sets
 *
 * @export
 * @class ZoweDatasetNode
 * @extends {vscode.TreeItem}
 */
class ZoweDatasetNode extends ZoweTreeNode_1.ZoweTreeNode {
    /**
     * Creates an instance of ZoweDatasetNode
     *
     * @param {string} label - Displayed in the [TreeView]
     * @param {vscode.TreeItemCollapsibleState} mCollapsibleState - file/folder
     * @param {ZoweDatasetNode} mParent
     * @param {Session} session
     */
    constructor(label, collapsibleState, mParent, session, contextOverride, etag, profile) {
        super(label, collapsibleState, mParent, session, profile);
        this.etag = etag;
        this.pattern = "";
        this.dirty = extension.ISTHEIA; // Make sure this is true for theia instances
        this.children = [];
        if (contextOverride) {
            this.contextValue = contextOverride;
        }
        else if (collapsibleState !== vscode.TreeItemCollapsibleState.None) {
            this.contextValue = extension.DS_PDS_CONTEXT;
        }
        else if (mParent && mParent.getParent()) {
            this.contextValue = extension.DS_MEMBER_CONTEXT;
        }
        else {
            this.contextValue = extension.DS_DS_CONTEXT;
        }
        this.tooltip = this.label;
        utils.applyIcons(this);
    }
    /**
     * Implements access to profile name
     * for {IZoweDatasetTreeNode}.
     *
     * @returns {string}
     */
    getProfileName() {
        return this.getProfile() ? this.getProfile().name : undefined;
    }
    /**
     * Retrieves child nodes of this ZoweDatasetNode
     *
     * @returns {Promise<ZoweDatasetNode[]>}
     */
    getChildren() {
        return __awaiter(this, void 0, void 0, function* () {
            if ((!this.pattern && this.contextValue === extension.DS_SESSION_CONTEXT)) {
                return [new ZoweDatasetNode(localize(0, null), vscode.TreeItemCollapsibleState.None, this, null, extension.INFORMATION_CONTEXT)];
            }
            if (this.contextValue === extension.DS_DS_CONTEXT ||
                this.contextValue === extension.DS_MEMBER_CONTEXT ||
                this.contextValue === extension.INFORMATION_CONTEXT) {
                return [];
            }
            if (!this.dirty || this.label === "Favorites") {
                return this.children;
            }
            if (!this.label) {
                vscode.window.showErrorMessage(localize(1, null));
                throw Error(localize(2, null));
            }
            // Gets the datasets from the pattern or members of the dataset and displays any thrown errors
            let responses = [];
            responses = yield vscode.window.withProgress({
                location: vscode.ProgressLocation.Notification,
                title: localize(3, null)
            }, () => {
                return this.getDatasets();
            });
            // push nodes to an object with property names to avoid duplicates
            const elementChildren = {};
            responses.forEach((response) => {
                // Throws reject if the Zowe command does not throw an error but does not succeed
                if (!response.success) {
                    throw Error(localize(4, null));
                }
                // Loops through all the returned dataset members and creates nodes for them
                for (const item of response.apiResponse.items) {
                    const existing = this.children.find((element) => element.label.trim() === item.dsname);
                    if (existing) {
                        elementChildren[existing.label] = existing;
                        // Creates a ZoweDatasetNode for a PDS
                    }
                    else if (item.dsorg === "PO" || item.dsorg === "PO-E") {
                        const temp = new ZoweDatasetNode(item.dsname, vscode.TreeItemCollapsibleState.Collapsed, this, null, undefined, undefined, this.getProfile());
                        elementChildren[temp.label] = temp;
                    }
                    else if (item.migr && item.migr.toUpperCase() === "YES") {
                        const temp = new ZoweDatasetNode(item.dsname, vscode.TreeItemCollapsibleState.None, this, null, extension.DS_MIGRATED_FILE_CONTEXT, undefined, this.getProfile());
                        elementChildren[temp.label] = temp;
                    }
                    else if (this.contextValue === extension.DS_SESSION_CONTEXT) {
                        // Creates a ZoweDatasetNode for a PS
                        const temp = new ZoweDatasetNode(item.dsname, vscode.TreeItemCollapsibleState.None, this, null, undefined, undefined, this.getProfile());
                        temp.command = { command: "zowe.ZoweNode.openPS", title: "", arguments: [temp] };
                        elementChildren[temp.label] = temp;
                    }
                    else {
                        // Creates a ZoweDatasetNode for a PDS member
                        const temp = new ZoweDatasetNode(item.member, vscode.TreeItemCollapsibleState.None, this, null, undefined, undefined, this.getProfile());
                        temp.command = { command: "zowe.ZoweNode.openPS", title: "", arguments: [temp] };
                        elementChildren[temp.label] = temp;
                    }
                }
            });
            this.dirty = false;
            if (Object.keys(elementChildren).length === 0) {
                return this.children = [new ZoweDatasetNode(localize(5, null), vscode.TreeItemCollapsibleState.None, this, null, extension.INFORMATION_CONTEXT)];
            }
            else {
                return this.children = Object.keys(elementChildren).sort().map((labels) => elementChildren[labels]);
            }
        });
    }
    getSessionNode() {
        return this.getParent() ? this.getParent().getSessionNode() : this;
    }
    /**
     * Returns the [etag] for this node
     *
     * @returns {string}
     */
    getEtag() {
        return this.etag;
    }
    /**
     * Set the [etag] for this node
     *
     * @returns {void}
     */
    setEtag(etagValue) {
        this.etag = etagValue;
    }
    getDatasets() {
        return __awaiter(this, void 0, void 0, function* () {
            const responses = [];
            try {
                if (this.contextValue === extension.DS_SESSION_CONTEXT) {
                    this.pattern = this.pattern.toUpperCase();
                    // loop through each pattern
                    for (const pattern of this.pattern.split(",")) {
                        responses.push(yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(this.getProfile()).dataSet(pattern.trim(), { attributes: true }));
                    }
                }
                else {
                    // Check if node is a favorite
                    let label = this.label.trim();
                    if (this.label.startsWith("[")) {
                        label = this.label.substring(this.label.indexOf(":") + 1).trim();
                    }
                    responses.push(yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(this.getProfile()).allMembers(label, { attributes: true }));
                }
            }
            catch (err) {
                yield utils.errorHandling(err, this.label, localize(6, null) + `zowe.List`);
            }
            return responses;
        });
    }
}
exports.ZoweDatasetNode = ZoweDatasetNode;

//# sourceMappingURL=ZoweDatasetNode.js.map
