"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Profiles_1 = require("../Profiles");
const PersistentFilters_1 = require("../PersistentFilters");
const utils_1 = require("../utils");
const extension = require("../extension");
// tslint:disable-next-line: max-classes-per-file
class ZoweTreeProvider {
    constructor(persistenceSchema, mFavoriteSession) {
        this.persistenceSchema = persistenceSchema;
        this.mFavoriteSession = mFavoriteSession;
        // Event Emitters used to notify subscribers that the refresh event has fired
        this.mOnDidChangeTreeData = new vscode.EventEmitter();
        this.onDidChangeTreeData = this.mOnDidChangeTreeData.event;
        this.createOwner = new utils_1.OwnerFilterDescriptor();
        this.validProfile = -1;
        this.mHistory = new PersistentFilters_1.PersistentFilters(this.persistenceSchema);
    }
    /**
     * Takes argument of type IZoweTreeNode and returns it converted to a general [TreeItem]
     *
     * @param {IZoweTreeNode} element
     * @returns {vscode.TreeItem}
     */
    getTreeItem(element) {
        return element;
    }
    getParent(element) {
        return element.getParent();
    }
    /**
     * Selects a specific item in the tree view
     *
     * @param {IZoweTreeNode}
     */
    setItem(treeView, item) {
        treeView.reveal(item, { select: true, focus: true });
    }
    /**
     * Called whenever the tree needs to be refreshed, and fires the data change event
     *
     */
    refreshElement(element) {
        element.dirty = true;
        this.mOnDidChangeTreeData.fire(element);
    }
    /**
     * Called whenever the tree needs to be refreshed, and fires the data change event
     *
     */
    refresh() {
        this.mOnDidChangeTreeData.fire();
    }
    /**
     * Change the state of an expandable node
     * @param provider the tree view provider
     * @param element the node being flipped
     * @param isOpen the intended state of the the tree view provider, true or false
     */
    flipState(element, isOpen = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (element.label !== "Favorites") {
                let usrNme;
                let passWrd;
                let baseEncd;
                let sesNamePrompt;
                if (element.contextValue.endsWith(extension.FAV_SUFFIX)) {
                    sesNamePrompt = element.label.substring(1, element.label.indexOf("]"));
                }
                else {
                    sesNamePrompt = element.label;
                }
                if ((!element.getSession().ISession.user) || (!element.getSession().ISession.password)) {
                    try {
                        const values = yield Profiles_1.Profiles.getInstance().promptCredentials(sesNamePrompt);
                        if (values !== undefined) {
                            usrNme = values[0];
                            passWrd = values[1];
                            baseEncd = values[2];
                        }
                    }
                    catch (error) {
                        vscode.window.showErrorMessage(error.message);
                    }
                    if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                        element.getSession().ISession.user = usrNme;
                        element.getSession().ISession.password = passWrd;
                        element.getSession().ISession.base64EncodedAuth = baseEncd;
                        this.validProfile = 1;
                    }
                    else {
                        return;
                    }
                    yield this.refreshElement(element);
                    yield this.refresh();
                }
                else {
                    this.validProfile = 1;
                }
            }
            else {
                this.validProfile = 1;
            }
            if (this.validProfile === 1) {
                element.iconPath = utils_1.applyIcons(element, isOpen ? extension.ICON_STATE_OPEN : extension.ICON_STATE_CLOSED);
                element.dirty = true;
                this.mOnDidChangeTreeData.fire(element);
            }
        });
    }
    onDidChangeConfiguration(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (e.affectsConfiguration(this.persistenceSchema)) {
                const setting = Object.assign({}, vscode.workspace.getConfiguration().get(this.persistenceSchema));
                if (!setting.persistence) {
                    setting.favorites = [];
                    setting.history = [];
                    yield vscode.workspace.getConfiguration().update(this.persistenceSchema, setting, vscode.ConfigurationTarget.Global);
                }
            }
        });
    }
    addHistory(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            if (criteria) {
                this.mHistory.addHistory(criteria);
                this.refresh();
            }
        });
    }
    deleteSessionByLabel(revisedLabel) {
        if (revisedLabel.includes("[")) {
            revisedLabel = revisedLabel.substring(0, revisedLabel.indexOf(" ["));
        }
        this.mHistory.removeSession(revisedLabel);
        this.refresh();
    }
}
exports.ZoweTreeProvider = ZoweTreeProvider;

//# sourceMappingURL=ZoweTreeProvider.js.map
