"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * This decorator can be applied to a class within a __mocks__ folder so that
 * a typescript class can be more easily mocked.
 *
 * @example <caption>Class in __mocks__<caption>
 * export class TestClass {
 *   // This method will be wrapped in a jest.fn() call with the implementation
 *   // being the contents of the below function.
 *   @MockMethod()
 *   public test() {
 *     return "string";
 *   }
 * }
 *
 * // In some other file
 * const myClass = new TestClass();
 * console.log(myClass.test()); // Prints "string" to the console
 * expect(myClass.test).toHaveBeenCalledTimes(2);
 * // Fails with message
 * // Expected mock function to have been called two times, but it was called one time
 */
function MockMethod() {
    return (target, key, descriptor) => {
        if (descriptor === undefined) {
            descriptor = Object.getOwnPropertyDescriptor(target, key);
        }
        const originalMethod = descriptor.value;
        descriptor.value = jest.fn(function () {
            originalMethod.apply(this, arguments);
        });
        return descriptor;
    };
}
exports.MockMethod = MockMethod;

//# sourceMappingURL=MockMethod.js.map
